#ifndef _KK_NEWCCU_MSG_H
#define _KK_NEWCCU_MSG_H

#include "kk_ccu_msg.h"

cJSON *property_info_build(const char *msgtype,const char *productCode,const char *deviceCode);
cJSON *property_payload_build(const char*method,const char *msgId,const char *version,cJSON *params);
cJSON * property_set(const char *productCode,const char *deviceCode,const char *msgId,const char *version,cJSON *params);
cJSON * property_report(const char *productCode,const char *deviceCode,const char *msgId,const char *version,cJSON *params);





int kk_ipc_send_json(cJSON *root);
cJSON * scene_execute(const char *sceneId);











#endif


